--
-- Pickup
-- Specialisation for lift and lower the pickup, ground ref, pressure pad and block the pickup if its overloaded
--
-- @author  Stefan Maurus
-- @date  04/06/14

-- Copyright © Stefan Maurus, www.stefanmaurus.de

-- Includes parts of the script from fruktor to block the pickup (source: ClaasQuadrant1200.lua)

--[[
    <pickup>
        <animation name="ANIMATIONNAME" rake="INDEX" groundRef="INDEX" /> <!--groundRef has to be on the buttom of the pickup-->
        <pressurePad index="INDEX" minRot="DEGREES" maxRot="DEGREES"/>
        <dirt wheat="INDEX" grass="INDEX" cleanNode="INDEX"/> <!--You have to be near cleanNode to clean the pickup -->
        <blockSound file="PATH" volume="1"/>
        <grassCleaningParticle node="INDEX" position="0 0 0" rotation="0 0 0" file="PATH"/>
        <wheatCleaningParticle node="INDEX" position="0 0 0" rotation="0 0 0" file="PATH"/>
    </pickup>
]]

Pickup = {};

function Pickup.prerequisitesPresent(specializations)
    return true;
end;

function Pickup:load(savegame)
    self.getIsLoadingFromAir = Pickup.getIsLoadingFromAir;

    self.setIsBlocked = SpecializationUtil.callSpecializationsFunction("setIsBlocked");
    self.isBlocked = false;
    self.blockTimer = 0;
    self.blockMaxTime = 1500;
    local name = getXMLString(self.xmlFile, "vehicle.pickup.blockSound#file");
    self.blockSound = createSample("blockSound");
    loadSample(self.blockSound, Utils.getFilename(name, self.baseDirectory), false);
    self.blockSoundVolume = getXMLFloat(self.xmlFile, "vehicle.pickup.blockSound#volume");

    self.loadedMaxLoad = getXMLFloat(self.xmlFile, "vehicle.pickup#maxload");
    self.maxLoad = self.loadedMaxLoad;

    self.pudirt = {};
    self.pudirt.grass = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pickup.dirt#grass"));
    self.pudirt.wheat = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pickup.dirt#wheat"));
    self.pucleanNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pickup.dirt#cleanNode"));
    self.cleaningEndTime = 0;
    setVisibility( self.pudirt.grass, false );
    setVisibility( self.pudirt.wheat, false );

    local psPath = string.format("vehicle.pickup.grassCleaningParticle");
    local cleanPSGrass = {};
    local particleNode = Utils.loadParticleSystem(self.xmlFile, cleanPSGrass, psPath, self.components, false, "$data/vehicles/particleSystems/trailerDischargeParticleSystem.i3d", self.baseDirectory);
    self.cleanPSGrass = cleanPSGrass;

    psPath = string.format("vehicle.pickup.wheatCleaningParticle");
    local cleanPSWheat = {};
    local particleNode = Utils.loadParticleSystem(self.xmlFile, cleanPSWheat, psPath, self.components, false, "$data/vehicles/particleSystems/trailerDischargeParticleSystem.i3d", self.baseDirectory);
    self.cleanPSWheat = cleanPSWheat;

    local uiScale = g_uiScale;
    if g_gameSettings ~= nil then
        uiScale = g_gameSettings:getValue("uiScale")
    end
    -- local vehicleHudBgWidth, vehicleHudBgHeight = getNormalizedScreenValues(225*uiScale, 125*uiScale)
    -- self.vehicleHudBg = Overlay:new("vehicleHudBg", g_baseUIFilename, 1-g_safeFrameOffsetX-vehicleHudBgWidth, g_safeFrameOffsetY, vehicleHudBgWidth, vehicleHudBgHeight);
    -- self.vehicleHudBg:setUVs(g_colorBgUVs);
    -- self.vehicleHudBg:setColor(unpack(g_colorBg));



    self.hud2 = {};
    local width, height = getNormalizedScreenValues(225*uiScale, 20*uiScale);
    local w, h = getNormalizedScreenValues(0, 125*uiScale);
    self.hud2.xPos = 1-g_safeFrameOffsetX-width;
    self.hud2.yPos = h+g_safeFrameOffsetY;
    self.hud2.hudWidth = width;
    self.hud2.overlay_hud     = Overlay:new("loadHUD_hud",     Utils.getFilename("hud/hud_block.dds", self.baseDirectory), self.hud2.xPos, self.hud2.yPos, width, height);
    self.hud2.overlay_bar     = Overlay:new("loadHUD_bar",     Utils.getFilename("hud/hud_barY.dds", self.baseDirectory), self.hud2.xPos, self.hud2.yPos, width, height);

    self.getIsAreaActive = Utils.overwrittenFunction(self.getIsAreaActive, Pickup.getIsAreaActive);

    self.pickup = {};
    self.pickup.pressurePad = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.pickup.pressurePad#index"));
    self.pickup.pressurePadMin = math.rad(getXMLFloat(self.xmlFile, "vehicle.pickup.pressurePad#minRot"));
    self.pickup.pressurePadMax = math.rad(getXMLFloat(self.xmlFile, "vehicle.pickup.pressurePad#maxRot"));

    local name = getXMLString(self.xmlFile, "vehicle.pickup.lowerSound#file");
    self.lowerSound = createSample("lowerSound");
    loadSample(self.lowerSound, Utils.getFilename(name, self.baseDirectory), false);
    self.lowerSoundVolume = getXMLFloat(self.xmlFile, "vehicle.pickup.lowerSound#volume");

    self.time = 0;

    self.pickUpSoundPlayed = false;

    self.fillLevelDeltas = {};
    self.currentActLoad = 0;
    self.showActLoad = 0;
    self.lastFillLevelDelta = 0;

    self.currentStep = 1;

    self.timeSinceLastLoad = 0;

    self.calcShownActLoad = SpecializationUtil.callSpecializationsFunction("calcShownActLoad");
    self.setShownActLoad = SpecializationUtil.callSpecializationsFunction("setShownActLoad");
end;

function Pickup:delete()
    Utils.deleteParticleSystem(self.cleanPSGrass);
    Utils.deleteParticleSystem(self.cleanPSWheat);
end;

function Pickup:readStream(streamId, connection)
    self.isBlocked = streamReadBool(streamId);
    self:setShownActLoad(streamReadFloat32(streamId), true);
end;

function Pickup:writeStream(streamId, connection)
    streamWriteBool(streamId, self.isBlocked);
    streamWriteFloat32(streamId, self.showActLoad);
end;

function Pickup:getIsAreaActive(superFunc, area)
    if superFunc ~= nil then
        return superFunc(self, area) and self.movingDirection > 0 and self.isPickupLowered; --just load when pickup is down and moving forward
    end;
    return self.movingDirection > 0 and self.isPickupLowered;
end;

function Pickup:readUpdateStream(streamId, connection, dirtyMask)
end;

function Pickup:writeUpdateStream(streamId, connection, dirtyMask)
end;

function Pickup:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Pickup:keyEvent(unicode, sym, modifier, isDown)
end;

function Pickup:update(dt)
    if self.isClient then
        if self:getIsActiveForSound() then
            if self.isPickupLowered and not self.pickUpSoundPlayed then
                self.pickUpSoundPlayed = true;
                playSample(self.lowerSound, 1, self.lowerSoundVolume, 0);
            end;

            if not self.isPickupLowered and self.pickUpSoundPlayed then
                playSample(self.lowerSound, 1, self.lowerSoundVolume, 0);
                self.pickUpSoundPlayed = false;
            end;
        end;
    end;

    if self.cleaningEndTime > 0 then
        self.cleaningEndTime = math.max(self.cleaningEndTime-dt, 0);
    end;


    if self.isBlocked and self:getIsTurnedOn() then
        self:setIsTurnedOn(false);
    end;

    --#
    if g_currentMission.player ~= nil then
        local pvx, pvy, pvz = getWorldTranslation(g_currentMission.player.rootNode);
        local px, py, pz = getWorldTranslation(self.pucleanNode);
        local distance = Utils.vector3Length(px-pvx, py-pvy, pz-pvz);
        if distance < 2 then
            if self.isBlocked then
                g_currentMission:addHelpButtonText( g_i18n:getText("PICKUP_CLEAN"), InputBinding.IMPLEMENT_EXTRA, nil, GS_PRIO_NORMAL)
                if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
                    self:setIsBlocked(false);
                end;
            end;
        end;
    end;

    if self.showActLoad > 0 and not self.isBlocked then
        local _,y,z = getRotation(self.pickup.pressurePad);
        local minMax = self.pickup.pressurePadMax-self.pickup.pressurePadMin;
        local minMaxPer = minMax*(self.showActLoad/self.maxLoad);
        setRotation(self.pickup.pressurePad, (self.pickup.pressurePadMin+minMaxPer),y,z);
    elseif self.isBlocked then
        local _,y,z = getRotation(self.pickup.pressurePad);
        setRotation(self.pickup.pressurePad, self.pickup.pressurePadMax,y,z);
    else
        local _,y,z = getRotation(self.pickup.pressurePad);
        setRotation(self.pickup.pressurePad, self.pickup.pressurePadMin,y,z);
    end;
end;

function Pickup:calcShownActLoad(target)
    if target == self.showActLoad then
        return;
    end;

    local step = self.currentStep * math.abs(self.showActLoad-target)/6;

    if self.timeSinceLastLoad > 1000 then
        step = step * 10;
    end;

    if target > self.showActLoad then
        self:setShownActLoad(math.min(self.showActLoad + step, target));
    else
        self:setShownActLoad(math.max(self.showActLoad - step, target));
    end;
end;

function Pickup:setShownActLoad(act, noEventSend)
    self.showActLoad = math.max(math.min(act, self.maxLoad), 0);
    SetShownActLoadEvent.sendEvent(self, act, noEventSend);
end;

function Pickup:updateTick(dt)
    if self.blades.currentUse < 0.5 then
        self.maxLoad = self.loadedMaxLoad*(self.blades.currentUse+0.5);
    else
        self.maxLoad = self.loadedMaxLoad;
    end;

    Cylindered.updateMovingPart(self, self.activeDirtyMovingParts[1], true);
    Cylindered.updateMovingPart(self, self.activeDirtyMovingParts[2], true);
    Cylindered.updateMovingPart(self, self.activeDirtyMovingParts[3], true);
    Cylindered.updateMovingPart(self, self.activeDirtyMovingParts[4], true);

    if self.isServer then
        local delta = self:getUnitFillLevel(self.forageWagon.fillUnitIndex) - self.lastFillLevelDelta;

        if delta > 0 then
            self.timeSinceLastLoad = 0;
            table.insert(self.fillLevelDeltas, {fillLevel = self:getUnitFillLevel(self.forageWagon.fillUnitIndex), t = self.time});

            if table.getn(self.fillLevelDeltas) == 6 then
                local newDelta = self.fillLevelDeltas[6].fillLevel - self.fillLevelDeltas[1].fillLevel;
                local newTime = self.fillLevelDeltas[6].t - self.fillLevelDeltas[1].t;

                normalizedDelta = newDelta/newTime*1000;
                self.currentActLoad = normalizedDelta;

                self.fillLevelDeltas = {};
            end;
        else
            self.timeSinceLastLoad = self.timeSinceLastLoad + dt;
        end;

        if (self.timeSinceLastLoad > 1000 and self.currentActLoad > 0) or self:getIsLoadingFromAir() then
            self.currentActLoad = 0;
        end;

        self.time = self.time + dt;

        self.lastFillLevelDelta = self:getUnitFillLevel(self.forageWagon.fillUnitIndex);

        self:calcShownActLoad(self.currentActLoad);

        if self.showActLoad == self.maxLoad and self.lastSpeed*3600 > 5 then
            self.blockTimer = self.blockTimer + dt;
            if self.blockTimer > self.blockMaxTime then
                if not self.isBlocked then
                    self.blockTimer = 0;
                    self:setIsBlocked(true);
                    self.isFirstRun2 = true;
                end
            end;
        else
            self.blockTimer = math.max(0, self.blockTimer - dt);
        end;
    end;

    if self.isClient then
        if self.cleaningEndTime > 0 then
            if self:getUnitFillType(self.forageWagon.fillUnitIndex) == FillUtil.fillTypeNameToInt["grass_windrow"] then
                Utils.setEmittingState(self.cleanPSGrass, true);
            else
                Utils.setEmittingState(self.cleanPSWheat, true);
            end;
        else
            Utils.setEmittingState(self.cleanPSGrass, false);
            Utils.setEmittingState(self.cleanPSWheat, false);
        end;
    end;
end;

function Pickup:draw()
    setTextColor(1.0, 1.0, 1.0, 1.0);

    setTextBold(false);
    setTextAlignment(RenderText.ALIGN_LEFT);

    local percentage = math.min(self.showActLoad/self.maxLoad, 1);

    if percentage > 0.95 then
        setOverlayColor(self.hud2.overlay_bar.overlayId, 1, 0, 0, 1)
    else
        setOverlayColor(self.hud2.overlay_bar.overlayId, 0, 1, 0, 1)
    end;

    self.hud2.overlay_bar.width = self.hud2.hudWidth * percentage;
    setOverlayUVs(self.hud2.overlay_bar.overlayId, 0, 0.05, 0, 1, percentage, 0.05, percentage, 1);
    self.hud2.overlay_bar:render();

    self.hud2.overlay_hud:render();

    setTextAlignment(RenderText.ALIGN_CENTER);

    local uiScale = g_uiScale;
    if g_gameSettings ~= nil then
        uiScale = g_gameSettings:getValue("uiScale")
    end
    local x, y = getNormalizedScreenValues(225*uiScale/2, 7);
    renderText(self.hud2.xPos+x, self.hud2.yPos+y, 0.012*uiScale, g_i18n:getText("ACT_LOAD").." "..string.format("%5.2f %%", 100*percentage));
    setTextAlignment(RenderText.ALIGN_LEFT);

    if self.isBlocked then
        g_currentMission:showBlinkingWarning(g_i18n:getText("PICKUP_IS_BLOCKED"), 7000);
    end;
end;

function Pickup:getIsLoadingFromAir()
    if self:getIsTurnedOn() or self.topRopesAreVis then
        return false;
    end;
    return true;
end;

function Pickup:setIsBlocked(state, noEventSend)
    SetBlockedEvent.sendEvent(self, state, noEventSend);
    self.isBlocked = state;
    if state == true then
        if self.isClient and self:getIsActiveForSound() then
            playSample(self.blockSound, 1, self.blockSoundVolume, 0);
        end;
        --if self.
        if self:getUnitFillType(self.forageWagon.fillUnitIndex) == FillUtil.fillTypeNameToInt["grass_windrow"] then
            setVisibility( self.pudirt.grass, true);
        else
            setVisibility( self.pudirt.wheat, true);
        end;
    else
        setVisibility( self.pudirt.grass, false);
        setVisibility( self.pudirt.wheat, false);
        self.cleaningEndTime = 300;
    end
end;

SetBlockedEvent = {};
SetBlockedEvent_mt = Class(SetBlockedEvent, Event);

InitEventClass(SetBlockedEvent, "SetBlockedEvent");

function SetBlockedEvent:emptyNew()
    local self = Event:new(SetBlockedEvent_mt);
    self.className="SetBlockedEvent";
    return self;
end;

function SetBlockedEvent:new(vehicle, state)
    local self = SetBlockedEvent:emptyNew()
    self.vehicle = vehicle;
    self.state = state;
    return self;
end;

function SetBlockedEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.state = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function SetBlockedEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
    streamWriteBool(streamId, self.state);
end;

function SetBlockedEvent:run(connection)
    self.vehicle:setIsBlocked(self.state, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetBlockedEvent:new(self.vehicle, self.state), nil, connection, self.object);
    end;
end;

function SetBlockedEvent.sendEvent(vehicle, state, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(SetBlockedEvent:new(vehicle, state), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(SetBlockedEvent:new(vehicle, state));
        end;
    end;
end;

SetShownActLoadEvent = {};
SetShownActLoadEvent_mt = Class(SetShownActLoadEvent, Event);

InitEventClass(SetShownActLoadEvent, "SetShownActLoadEvent");

function SetShownActLoadEvent:emptyNew()
    local self = Event:new(SetShownActLoadEvent_mt);
    self.className="SetShownActLoadEvent";
    return self;
end;

function SetShownActLoadEvent:new(vehicle, showActLoad)
    local self = SetShownActLoadEvent:emptyNew()
    self.vehicle = vehicle;
    self.showActLoad = showActLoad;
    return self;
end;

function SetShownActLoadEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);

    self.showActLoad = streamReadFloat32(streamId);
    if self.vehicle ~= nil then
        self.vehicle:setShownActLoad(self.showActLoad, true);
    end;
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetShownActLoadEvent:new(self.vehicle, self.showActLoad), nil, connection, self.vehicle);
    end;
end;

function SetShownActLoadEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
    streamWriteFloat32(streamId, self.showActLoad);
end;

function SetShownActLoadEvent.sendEvent(vehicle, showActLoad, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(SetShownActLoadEvent:new(vehicle, showActLoad), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(SetShownActLoadEvent:new(vehicle, showActLoad));
        end;
    end;
end;
-- converted